#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <vector>
#include <iostream>
#include <fstream>
#include <GL/glut.h>
#include <GL/gl.h>
#include <assert.h>
#include <map>
#include <windows.h>

using namespace std;


typedef struct {
  float x;
  float y;
  float z;
  
} Normal;


//data strucutre to store 3d points
typedef struct {
  float x;
  float y;
  float z;
  Normal normal;
  
} Vector3;

//data structure to store triangle - note that v1, v2, and v3 are indexes into the vertex array
typedef struct {
  int v1;
  int v2;
  int v3;
  Vector3 normal;
  Vector3 color;
} Tri;

//Edge datastructure
typedef struct Edge{
  int R_tri;  //starts counting at 0 - do not decrement
  int L_tri;  //starts counting at 0 - do not decrement
  int v1; //starts counting at 1 like other vertex index
  int v2; //starts counting at 1 like other vertex index
  Vector3 ref;
  Vector3 normal;
} Edge;

//all the triangles in the mesh
vector<Tri> Triangles;
//all the vertices in the mesh
vector<Vector3> Vertices;
//all the edges in the mesh
vector<Edge> Edges;
//auxiliary data structure just used to create Edges - you do not need to reference this
map<unsigned int, Edge> MapEdges;

vector<Edge> silEdges;

vector<Edge> creaseEdges;

bool file_exists;



//for computing the center point
//and extent of the model
Vector3 center;
float max_x, max_y, max_z, min_x, min_y, min_z;
float max_extent;

//variables for virtual trackball
int track;
int startx, starty, endx, endy;
float angle;
Vector3 axis;
int display_mode;
int view_mode;
int lighting_mode;

GLfloat objectM[4][4] = {
  {1.0, 0.0, 0.0, 0.0},
  {0.0, 1.0, 0.0, 0.0},
  {0.0, 0.0, 1.0, 0.0},
  {0.0, 0.0, 0.0, 1.0}
};

GLfloat *trackballM = (GLfloat *)objectM;

GLfloat cameraM[4][4] = {
  {1.0, 0.0, 0.0, 0.0},
  {0.0, 1.0, 0.0, 0.0},
  {0.0, 0.0, 1.0, 0.0},
  {0.0, 0.0, 0.0, 1.0}
};

GLfloat *camM = (GLfloat *)cameraM;

///globals for reshape
int GW;
int GH;

/***************
* Lab 8 variables
*/

float o_tx, o_ty;
float o_zx, o_zy;
float o_cx, o_cy;

float tx, ty;
float zx, zy;
float cx, cy;

float o_lx, o_ly;
float lx, ly;
float * light_x;
float * light_y;

float phi, theta;
float ex, ey, ez, ux, uy, uz, ax, ay, az;
int mode, c_mode;
bool change_view, draw_norms, first_pos, shape_done, settings_change;

float g_theta, rx, ry, rz;

float old_scale, f_scale;
float u_scale;

int DRAW_TYPE;

int light;

//globals for lighting - use a white light and apply materials
GLfloat light_pos[4] = {1.0, 1.0, 1.5, 1.0};
GLfloat light_amb[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_diff[4] = {0.6, 0.6, 0.6, 1.0};
GLfloat light_spec[4] = {0.8, 0.8, 0.8, 1.0};

//kwarm = 0.8, 0.6, 0.6
//kcool = 0.4, 0.4, 0.7
//kwarm - kcool = 0.4, 0.2, -0.1
//kcool - kwarm = -0.4, -0.2, 0.1

//warm
GLfloat light_pos1[4] = {3.0, 3.0, 1.0, 0.0};
GLfloat light_amb1[4] = {0.6, 0.5, 0.65, 1.0};
GLfloat light_diff1[4] = {0.2, 0.1, -0.05, 1.0};
GLfloat light_spec1[4] = {0.0, 0.0, 0.0, 0.0};

Vector3 cool;

// cool
GLfloat light_pos2[4] = {-3.0, -3.0, 1.0, 0.0};
GLfloat light_amb2[4] = {0.6, 0.5, 0.65, 1.0};
GLfloat light_diff2[4] = {-0.2, -0.1, 0.05, 1.0};
GLfloat light_spec2[4] = {0.0, 0.0, 0.0, 0.0};

Vector3 cross(Vector3 a, Vector3 b)
{
	Vector3 c;
	c.x = a.y * b.z - a.z * b.y;
	c.y = a.z * b.x - a.x * b.z;
	c.z = a.x * b.y - a.y * b.x;
	return c; 
}

float dot (Vector3 a, Vector3 b)
{
	return a.x * b.x + a.y * b.y + a.z * b.z;
}


float mag(Vector3 a)
{
	return sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
}


//forward declarations of functions
void readLine(char* str);
void readStream(istream& is);
void drawTri(Tri);
void drawObjects();
void display();
void TrackBall();


float p2w_x(int p_x) {
  float x_i = ( (float)p_x - ((GW-1.0)/2.0) )*2.0/GW;
  return(((float)GW/(float)GH)*x_i);
}

float p2w_y(int p_y) {
  return( ( (float)p_y - ((GH-1.0)/2.0) )*2.0/GH);
}

Vector3 unit(Vector3 a)
{
	Vector3 c;
	c.x = a.x / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.y = a.y / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.z = a.z / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	return c;
}

Normal unitN(Normal a)
{
	Normal c;
	c.x = a.x / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.y = a.y / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	c.z = a.z / sqrtf(a.x * a.x + a.y * a.y + a.z * a.z);
	return c;
}

//open the file for reading
void ReadFile(char* filename) {
  printf("Reading coordinates from %s\n", filename);
  
  ifstream in_f(filename);
  if (!in_f)  {
    printf("Could not open file %s\n", filename);
    //exit(-1);
  } else {
    readStream(in_f);
  }
  
}

//process the input stream from the file
void readStream(istream& is)
{
  char str[256];
  for (;is;) {
    is >> ws;
    is.get(str,sizeof(str));
    if (!is) break;
    is.ignore(9999,'\n');
    readLine(str);
  }
}

//process each line of input save vertices and faces appropriately
void readLine(char* str) {
  
  int indx = 0, vi;
  float x, y, z;
  float r, g, b;
  int mat;
  
  if (str[0]=='#') return;
  if (str[0]=='V' && !strncmp(str,"Vertex ",7)) {
    static Vector3 v;
    assert(sscanf(str,"Vertex %d %g %g %g",&vi,&x,&y,&z)==4);

    v.x = x;
    v.y = y;
    v.z = z;

    Vertices.push_back(v);
    //cout << "made a vertex " << v.x << " " << v.y << " " << v.z << endl;
    //house keeping to display in center of the scene
    center.x += v.x;
    center.y += v.y;
    center.z += v.z;
    if (v.x > max_x) max_x = v.x; if (v.x < min_x) min_x = v.x;
    if (v.y > max_y) max_y = v.y; if (v.y < min_y) min_y = v.y;
    if (v.z > max_z) max_z = v.z; if (v.z < min_z) min_z = v.z;
  } else if (str[0]=='F' && !strncmp(str,"Face ",5)) {
    Tri t;
    char* s=str+4;
    int fi=-1;
    for (int t_i = 0;;t_i++) {
      while (*s && isspace(*s)) s++;
      //if we reach the end of the line break out of the loop
      if (!*s) break;
      //save the position of the current character
      char* beg=s;
      //advance to next space
      while (*s && isdigit(*s)) s++;
      //covert the character to an integer
      int j=atoi(beg);
      //the first number we encounter will be the face index, don't store it
      if (fi<0) { fi=j; continue; }
      //otherwise process the digit we've grabbed in j as a vertex index
      //the first number will be the face id the following are vertex ids
      if (t_i == 1)
        t.v1 = j;
      else if (t_i == 2)
        t.v2 = j;
      else if (t_i == 3)
        t.v3 = j;
      //if there is more data to process break out
      if (*s =='{') break;
    }
    //initialization - replace with real calculation

	Vector3 p;
	p.x = Vertices[t.v2-1].x - Vertices[t.v1-1].x;
	p.y = Vertices[t.v2-1].y - Vertices[t.v1-1].y;
	p.z = Vertices[t.v2-1].z - Vertices[t.v1-1].z;

	Vector3 q;
	q.x = Vertices[t.v3-1].x - Vertices[t.v1-1].x;
	q.y = Vertices[t.v3-1].y - Vertices[t.v1-1].y;
	q.z = Vertices[t.v3-1].z - Vertices[t.v1-1].z;


	t.normal = unit(cross(p, q));

	Vertices[t.v1-1].normal.x += t.normal.x;
	Vertices[t.v1-1].normal.y += t.normal.y;
	Vertices[t.v1-1].normal.z += t.normal.z;
	
	Vertices[t.v1-1].normal = unitN(Vertices[t.v1-1].normal);

	Vertices[t.v2-1].normal.x += t.normal.x;
	Vertices[t.v2-1].normal.y += t.normal.y;
	Vertices[t.v2-1].normal.z += t.normal.z;
	
	Vertices[t.v2-1].normal = unitN(Vertices[t.v2-1].normal);

	Vertices[t.v3-1].normal.x += t.normal.x;
	Vertices[t.v3-1].normal.y += t.normal.y;
	Vertices[t.v3-1].normal.z += t.normal.z;
	
	Vertices[t.v3-1].normal = unitN(Vertices[t.v3-1].normal);


    //possibly process colors if the mesh has colors
    if (*s && *s =='{'){
      char *s1 = s+1;
      cout << "trying to parse color " << !strncmp(s1,"rgb",3) << endl;
      //if we're reading off a color
      if (!strncmp(s1,"rgb=",4)) {
        //grab the values of the string
        assert(sscanf(s1,"rgb=(%g %g %g) matid=%d",&r,&g,&b,&mat)==4);
        t.color.x = r; t.color.x = g; t.color.x = b;
        cout << "set color to: " << r << " " << g << " " << b << endl;
      }
    }
    Triangles.push_back(t);
    //cout << "made a tri " << t.v1 << " " << t.v2 << " " << t.v3 << endl;
  }
}



//drawing routine to draw triangles as wireframe
void drawTria(Tri t) {
  if(display_mode == 0) {

    glBegin(DRAW_TYPE);

	glNormal3f(t.normal.x, t.normal.y, t.normal.z);
    glColor3f(0.0, 0.0, 0.5);
    //note that the vertices are indexed starting at 0, but the triangles
    //index them starting from 1, so we must offset by -1!!!
    glVertex3f(Vertices[t.v1 - 1].x, 
      Vertices[t.v1 - 1].y,
      Vertices[t.v1 - 1].z);
    glVertex3f(Vertices[t.v2 - 1].x, 
      Vertices[t.v2 - 1].y,
      Vertices[t.v2 - 1].z);
    glVertex3f(Vertices[t.v3 - 1].x, 
      Vertices[t.v3 - 1].y,
      Vertices[t.v3 - 1].z);
    glEnd();
  }

    if(display_mode == 1) {

    glBegin(DRAW_TYPE);

    glColor3f(0.0, 0.0, 0.5);
    //note that the vertices are indexed starting at 0, but the triangles
    //index them starting from 1, so we must offset by -1!!!


   Vector3 cur;

   cur = Vertices[t.v1-1];

   glNormal3f(cur.normal.x, cur.normal.y, cur.normal.z);
		glVertex3f(Vertices[t.v1 - 1].x, 
			Vertices[t.v1 - 1].y,
			Vertices[t.v1 - 1].z);

	cur = Vertices[t.v2-1];
    glNormal3f(cur.normal.x, cur.normal.y, cur.normal.z);
		glVertex3f(Vertices[t.v2 - 1].x, 
			Vertices[t.v2 - 1].y,
			Vertices[t.v2 - 1].z);

	cur = Vertices[t.v3-1];
	glNormal3f(cur.normal.x, cur.normal.y, cur.normal.z);
		glVertex3f(Vertices[t.v3 - 1].x, 
			Vertices[t.v3 - 1].y,
			Vertices[t.v3 - 1].z);
    glEnd();
  }

  if (draw_norms)
  {
	Vector3 tail;
	tail.x = (Vertices[t.v1-1].x + Vertices[t.v2 - 1].x + Vertices[t.v3 - 1].x)/3.0;
	tail.y = (Vertices[t.v1-1].y + Vertices[t.v2 - 1].y + Vertices[t.v3 - 1].y)/3.0;
	tail.z = (Vertices[t.v1-1].z + Vertices[t.v2 - 1].z + Vertices[t.v3 - 1].z)/3.0;

	float scale_factor = 1;
	float l_normal = sqrtf(t.normal.x * t.normal.x + t.normal.y * t.normal.y + t.normal.z * t.normal.z);

		scale_factor = .01;
	Vector3 head;
	head.x = tail.x + scale_factor * t.normal.x;
	head.y = tail.y + scale_factor * t.normal.y;
	head.z = tail.z + scale_factor * t.normal.z;

	
	glPointSize(2.0);
	glBegin(GL_LINES);
		glColor3f(1.0, 0.0, 0.0);
		glVertex3f(tail.x, tail.y, tail.z);
		glVertex3f(head.x, head.y, head.z);
	glEnd();
	
  }


}


void DrawAllEdges() {
  
  glColor3f(1.0, 0.0, 1.0);
  for (int k=0; k < Edges.size(); k++) {
    glBegin(GL_LINES);
    glVertex3f(Vertices[Edges[k].v1-1].x, Vertices[Edges[k].v1-1].y, Vertices[Edges[k].v1-1].z);
    glVertex3f(Vertices[Edges[k].v2-1].x, Vertices[Edges[k].v2-1].y, Vertices[Edges[k].v2-1].z);
    glEnd();
  }
  
}

void DrawCreaseEdges() {
  
  glColor3f(0.0, 0.0, 0.0);

  float offset = 0.001;
  Vector3 raise, norm, cam, p, q, ucool;
  float width = 1.0;
    Tri l, r;

  	Vector3 mid;

  Edge e;

  for (int k=0; k < creaseEdges.size(); k++) {

	  e = creaseEdges[k];
	
	  norm = e.normal;

	  	l = Triangles[e.L_tri];
		r = Triangles[e.R_tri];
		norm.x = l.normal.x + r.normal.x;
		norm.y = l.normal.y + r.normal.y;
		norm.z = l.normal.z + r.normal.z;
		norm = unit(norm);

	  	p = Vertices[e.v1-1];
		q = Vertices[e.v2-1];

		cool.x = -1 + sqrtf(3) * cos(phi) * cos(theta);;
		cool.y = -1 + sqrtf(3) * sin(phi);
		cool.z = 1 +  sqrtf(3) * cos(phi) * sin(theta);;

		
	mid.x = (p.x + q.x) / 2;
	mid.y = (p.y + q.y) / 2;
	mid.z = (p.z + q.z) / 2;

	  raise = e.ref;
	  mid = unit(mid);
	  ucool = unit(cool);
	  cam.x = -ucool.x + mid.x;
	  cam.y = -ucool.y + mid.y;
	  cam.z = -ucool.z + mid.z;

	float theta_1, acos_1;
	acos_1 = dot(cam, norm) / (mag(cam) * mag(norm));
	theta_1 = acosf(acos_1);
	double PI = 3.14159265358979323846;
	width = theta_1 * 1.5;

  raise.x = raise.x * offset;
  raise.y = raise.y * offset;
  raise.z = raise.z * offset;

  //glNormal3f(creaseEdges[k].normal.x, creaseEdges[k].normal.y, creaseEdges[k].normal.z);
  glLineWidth(width);
    glBegin(GL_LINES);
	glVertex3f(Vertices[creaseEdges[k].v1-1].x - raise.x, Vertices[creaseEdges[k].v1-1].y - raise.y, Vertices[creaseEdges[k].v1-1].z - raise.z);
    glVertex3f(Vertices[creaseEdges[k].v2-1].x - raise.x, Vertices[creaseEdges[k].v2-1].y - raise.y, Vertices[creaseEdges[k].v2-1].z - raise.z);
    glEnd();
  }
  
}

void findCreaseEdges()
{

	Vector3 p, q;
	Tri l, r;
	Edge e;
	Vector3 mid;
	Vector3 ref;
	creaseEdges.clear();

	for (int k=0; k < Edges.size(); k++) {
    e = Edges[k];
	l = Triangles[e.L_tri];
	r = Triangles[e.R_tri];
	p = Vertices[e.v1-1];
	q = Vertices[e.v2-1];

	mid.x = (p.x + q.x) / 2;
	mid.y = (p.y + q.y) / 2;
	mid.z = (p.z + q.z) / 2;

	Vector3 temp;

	temp.x = -ex - mid.x;
	temp.y = -ey - mid.y;
	temp.z = -ez - mid.z;

	ref = unit(temp);

	float theta_1, acos_1;
	acos_1 = dot(l.normal, r.normal) / (mag(r.normal) * mag(l.normal));
	theta_1 = acosf(acos_1);
	double PI = 3.14159265358979323846;


	//Raise Edges;
	Edges[k].ref = ref;

	//compute edge normal;
	Vector3 norm;
	norm.x = l.normal.x + r.normal.x;
	norm.y = l.normal.y + r.normal.y;
	norm.z = l.normal.z + r.normal.z;

	e.normal = unit(norm);

	float thresh = PI / 12;
	if ((theta_1 >= PI / 2 - thresh) && (theta_1 <= 3 * PI / 2 + thresh))
	{
		creaseEdges.push_back(Edges[k]);
	}

  }


}

void DrawSilEdges() {
  
  glColor3f(0.0, 0.0, 0.0);

  Tri l, r;
  float offset = 0.001;
  Vector3 raise, cam, mid, norm, p, q, ucool;
  float width = 1;
  Edge e;

  for (int k=0; k < silEdges.size(); k++) {
	  raise = silEdges[k].ref;

	  	e = silEdges[k];

		l = Triangles[e.L_tri];
		r = Triangles[e.R_tri];
		norm.x = l.normal.x + r.normal.x;
		norm.y = l.normal.y + r.normal.y;
		norm.z = l.normal.z + r.normal.z;
		norm = unit(norm);
		
	  	p = Vertices[e.v1-1];
		q = Vertices[e.v2-1];

	mid.x = (p.x + q.x) / 2;
	mid.y = (p.y + q.y) / 2;
	mid.z = (p.z + q.z) / 2;

		cool.x = -1 + sqrtf(3) * cos(phi) * cos(theta);;
		cool.y = -1 + sqrtf(3) * sin(phi);
		cool.z = 1 + sqrtf(3) * cos(phi) * sin(theta);;
		

	  raise = e.ref;
	  mid = unit(mid);
	  ucool = unit(cool);
	  cam.x = -ucool.x + mid.x;
	  cam.y = -ucool.y + mid.y;
	  cam.z = -ucool.z + mid.z;

	float theta_1, acos_1;
	acos_1 = dot(cam, norm) / (mag(cam) * mag(norm));
	theta_1 = acosf(acos_1);
	double PI = 3.14159265358979323846;
		width = theta_1 * 1.5;

		//width = 4;


  raise.x = raise.x * offset;
  raise.y = raise.y * offset;
  raise.z = raise.z * offset;



  //glNormal3f(silEdges[k].normal.x, silEdges[k].normal.y, silEdges[k].normal.z);
glLineWidth(width);
    glBegin(GL_LINES);
	glVertex3f(Vertices[silEdges[k].v1-1].x - raise.x, Vertices[silEdges[k].v1-1].y - raise.y, Vertices[silEdges[k].v1-1].z - raise.z);
    glVertex3f(Vertices[silEdges[k].v2-1].x - raise.x, Vertices[silEdges[k].v2-1].y - raise.y, Vertices[silEdges[k].v2-1].z - raise.z);
    glEnd();
	//fill in gaps

  }

  
}

void findSilEdges()
{

	Vector3 p, q;
	Tri l, r;
	Edge e;
	Vector3 mid;
	Vector3 ref;
	silEdges.clear();

	for (int k=0; k < Edges.size(); k++) {
    e = Edges[k];
	l = Triangles[e.L_tri];
	r = Triangles[e.R_tri];
	p = Vertices[e.v1-1];
	q = Vertices[e.v2-1];

	mid.x = (p.x + q.x) / 2;
	mid.y = (p.y + q.y) / 2;
	mid.z = (p.z + q.z) / 2;

	Vector3 temp;

	temp.x = -ex - mid.x;
	temp.y = -ey - mid.y;
	temp.z = -ez - mid.z;

	ref = unit(temp);

	
	//Raise Edges;
	Edges[k].ref = ref;

	//compute edge normal;
	Vector3 norm;
	norm.x = l.normal.x + r.normal.x;
	norm.y = l.normal.y + r.normal.y;
	norm.z = l.normal.z + r.normal.z;

	e.normal = unit(norm);

	float theta_1, theta_2;
	theta_1 = dot(l.normal, ref) / (mag(ref) * mag(l.normal));
	theta_2 = dot(r.normal, ref) / (mag(ref) * mag(r.normal));

	if ((theta_1 >= 0 && theta_2 <= 0) || (theta_1 <= 0 && theta_2 >= 0))
	{
		silEdges.push_back(Edges[k]);
	}

  }

}


void reshape(int w, int h) {
  GW = w;
  GH = h;
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (view_mode == 0)
    glOrtho( -2.0*(float)w/h, 2.0*(float)w/h, -2.0, 2.0, 1.0, 15.0);
  //else... fill in
  else
	gluPerspective(30.0, (float)w/h, 1.0, 15.0);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, w, h);
  
  glutPostRedisplay();
  glutPostRedisplay();
  
}

int mat = 0;
//set up some materials
typedef struct materialStruct {
  GLfloat ambient[4];
  GLfloat diffuse[4];
  GLfloat specular[4];
  GLfloat shininess[1];
} materialStruct;

materialStruct surface1 = {
  {0.3, 0.0, 0.0, 1.0},
  {0.9, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};
materialStruct surface2 = {
  {0.5, 0.5, 0.5, 1.0},
  {1.0, 1.0, 1.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};
materialStruct surface3 = {
  {0.0, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0, 0.0, 0.0, 1.0},
  {0.0}
};


//sets up a specific material
void materials(materialStruct materials) {
  glMaterialfv(GL_FRONT, GL_AMBIENT, materials.ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, materials.diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, materials.specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, materials.shininess);
}


float t_ground = 0.0;

void drawObjects() {

	  t_ground = -(center.y) * 4;
  
  
  glPushMatrix();

  //transforms for the mesh
   glTranslatef(tx, ty, 0);

	float s = 1.5;
   glScalef(f_scale * s, f_scale * s, f_scale * s);

   glMultMatrixf(trackballM);

  //leave these transformations in as they center and scale each mesh correctly
  //scale object to window
  glScalef(1.0/(float)max_extent, 1.0/(float)max_extent, 1.0/(float)max_extent);
  //translate the object to the orgin


  glTranslatef(-(center.x), -(center.y), -(center.z));
  //draw the wireframe mesh
  for(int j = 0; j < Triangles.size(); j++) {
    //Add transforms, if needed here...

	  drawTria(Triangles[j]);
  }
	
  if (lighting_mode == 1)
  {
      materials(surface3);
	  findSilEdges();
	  DrawSilEdges();

	  findCreaseEdges();
	  DrawCreaseEdges();
  }
  	  materials(surface2);

  glPopMatrix();
}

//function to create the edge vector - assuming cw order
void computeEdge() {
  unsigned int hash_left;
  unsigned int hash_right;
  unsigned int hash;
  Edge e;
  int right = 0;
  map<unsigned int, Edge>::iterator map_it;
  Edge old_e;

  e.ref.x = 0.0;
  e.ref.y = 0.0;
  e.ref.z = 0.0;

  e.normal.x = 0.0;
  e.normal.y = 0.0;
  e.normal.z = 0.0;
  
  //for each triangle add itself to the edge map
  for(int j = 0; j < Triangles.size(); j++) {
    //for each of the edges
    for (int k=0; k < 3; k++) {
      if ( k == 0) {
        //compute hash for edge between v1 and v2
        if (Triangles[j].v1 < Triangles[j].v2) {
          e.v1 = hash_left = Triangles[j].v1;
          e.v2 = hash_right = Triangles[j].v2;
          e.R_tri = j;
          e.L_tri = -1;
          right = 1;
        } else {
          e.v1 = hash_left = Triangles[j].v2;
          e.v2 = hash_right = Triangles[j].v1;
          e.L_tri = j;
          e.R_tri = -1;
        }
      } else if (k == 1) {
        //compute hash for edge between v2 and v3
        if (Triangles[j].v2 < Triangles[j].v3) {
          e.v1 = hash_left = Triangles[j].v2;
          e.v2 = hash_right = Triangles[j].v3;
          e.R_tri = j;
          e.L_tri = -1;
          right = 1;
        } else {
          e.v1 = hash_left = Triangles[j].v3;
          e.v2 = hash_right = Triangles[j].v2;
          e.L_tri = j;
          e.R_tri = -1;
        }
      } else if (k == 2) {
        //compute hash for edge between v3 and v1
        if (Triangles[j].v3 < Triangles[j].v1) {
          e.v1 = hash_left = Triangles[j].v3;
          e.v2 = hash_right = Triangles[j].v1;
          e.R_tri = j;
          e.L_tri = -1;
          right = 1;
        } else {
          e.v1 = hash_left = Triangles[j].v1;
          e.v2 = hash_right = Triangles[j].v3;
          e.L_tri = j;
          e.R_tri = -1;
        }
      }
      if (e.v1 > e.v2) {
        cout << "order problem!!!" << endl;
      }
      //create the unique ID by shifting hash_right to the right
      hash_right <<= 16;
      //now include hash left
      hash = hash_left | hash_right;
      map_it = 0;
      //check if there is already an edge or not
      map_it = MapEdges.find(hash);
      if ((*map_it).first ==hash) {
        old_e = (*map_it).second;
        //make sure that this edge only has one tri defined
        if (old_e.L_tri == -1) {
          old_e.L_tri = j;
        } else if (old_e.R_tri == -1) {
          old_e.R_tri = j;
        } else {
          cout << "Problem already used hash!!!" << endl;
          assert(0);
        }
        //since we're done with this edge completely add it to the edge vector
        Edges.push_back(old_e);
      } else {
        //add this edge
        MapEdges[hash] = e;
      }
      right = 0;
    }
  }
  cout << "processed " << Edges.size() << " edges in map" << MapEdges.size() << endl;
  
}


void zoom()
{
	o_zx = zx;
	o_zy = zy;

	float di = 0.0;
	float dj = 0.0;

	//amount to translate

	di = p2w_x(endx * 2) - p2w_x(startx * 2);
	dj = p2w_y(endy * 2) - p2w_y(starty * 2);

	zx = zx + di;
	zy = zy + dj;

	if(zx + zy < -6 || zx + zy > 3)
	{
		zx = o_zx;
		zy = o_zy;
	}

	ez = -(zy + zx)  + 6;
	
	startx = endx;
    starty = endy;

}

void move_light()
{
	o_lx = lx;
	o_ly = ly;

	float di = 0.0;
	float dj = 0.0;

	//amount to translate

	di = p2w_x(endx * 2) - p2w_x(startx * 2);
	dj = p2w_y(endy * 2) - p2w_y(starty * 2);

	lx = lx + di;
	ly = ly + dj;

	if (lx > 10 || lx < -10)
	{
		lx = o_lx;
	}
	if (ly > 10 || ly < -10)
	{
		ly = o_ly;
	}
	if (shape_done)
	{
		lx = o_lx;
		ly = o_ly;
	}


	light_pos[0] = lx;
	light_pos[1] = ly;

    
	startx = endx;
    starty = endy;
}


void move()
{
	o_cx = cx;
	o_cy = cy;

	float di = 0.0;
	float dj = 0.0;

	//amount to translate

	di = p2w_x(endx * 2) - p2w_x(startx * 2);
	dj = p2w_y(endy * 2) - p2w_y(starty * 2);

	cx = cx + di;
	cy = cy + dj;

	if (cx > 2 || cx < -2)
	{
		cx = o_cx;
	}
	if (cy > 2 || cy < -2)
	{
		cy = o_cy;
	}
	if (shape_done)
	{
		cx = o_cx;
		cy = o_cy;
	}

	ex = ex -cx;
	ey = ey -cy;

	ax = -cx;
	ay = -cy;

    startx = endx;
    starty = endy;

}

void rotate()
{
	Vector3 a, b, c;
	a.x = p2w_x(startx) - tx / 2 - cx / 2;
	a.y = p2w_y(starty) - ty / 2 - cy / 2;

	float a_z = 1 - ((a.x * a.x) + (a.y * a.y));

	if (a_z > 0)
	{
		a.z = sqrtf(a_z);
	}
	else
	{
		
		a.x = a.x / sqrtf((a.x * a.x) + (a.y * a.y));
		a.y = a.y / sqrtf((a.x * a.x) + (a.y * a.y));
		a.z = 0;
	}

	b.x = p2w_x(endx) - tx / 2 - cx / 2;
	b.y = p2w_y(endy) - ty / 2 - cy / 2;
	
	
	float b_z = 1 - ((a.x * a.x) + (a.y * a.y));

	if (b_z > 0)
	{
		b.z = sqrtf(b_z);
	}
	else
	{
		b.x = b.x / sqrtf((b.x * b.x) + (b.y * b.y));
		b.y = b.y / sqrtf((b.x * b.x) + (b.y * b.y));
		b.z = 0;
	}

	c = cross(a, b);

	float m_a = sqrtf(dot(a, a));
	float m_b = sqrtf(dot(b, b));

	float theta = acosf (dot(a, b) / (mag(a) * mag(b)) );

	g_theta = 360 * theta/3.14159265358979323846;
	
	rx = c.x;
	ry = c.y;
	rz = c.z;
}

void trans()
{
	o_tx = tx;
	o_ty = ty; 
	
	float di = 0.0;
	float dj = 0.0;

		//amount to translate

	di = p2w_x(endx) - p2w_x(startx);
	dj = p2w_y(endy) - p2w_y(starty);

	tx = tx + 2 * di;
	ty = ty + 2 * dj;


	if (tx < -2 || tx > 2)
	{
		tx = o_tx;
	}
	if (ty < -2 || ty > 2)
	{
		ty = o_ty;
	}


	if (shape_done)
	{
		tx = o_tx;
		ty = o_ty;
	}

	startx = endx;
	starty = endy;
}

void revolve()
{

	float di = 0.0;
	float dj = 0.0;

		//amount to translate

	di = p2w_x(endx) - p2w_x(startx);
	dj = p2w_y(endy) - p2w_y(starty);

	theta += di;
	phi -= dj;

	float R = 6;

	ex = R * cos(phi) * cos(theta) - cx;
	ey = R * sin(phi) - cy;
	ez = R * cos(phi) * sin(theta);
	startx = endx;
	starty = endy;
}


void scale()
{

	if (first_pos)
	{
		f_scale = old_scale;
	}

	float di = 1.0;
	float dj = 1.0;
	float dk = 1.0;

	float unitX = p2w_x(startx) - tx / 2 - cx / 2;
	float unitY = p2w_y(starty) - ty / 2 - cy / 2;

	float unit = sqrtf( (unitX * unitX) + (unitY * unitY) );

	if (first_pos)
	{
		u_scale = unit;
	}

	float dx = p2w_x(endx) - tx / 2 - cx / 2;
	float dy = p2w_y(endy) - ty / 2 - cy / 2;

	float numerator = sqrtf( (dx * dx) + (dy * dy) );

	float scale = numerator / u_scale;

	float product = f_scale * scale;

	if (product > 4)
	{
		product = 4;
	}
	if  (product < 0.2)
	{
		product = .2;
	}

	if (!shape_done)
	{
		f_scale = product;
	}

	if (shape_done)
	{
		old_scale = f_scale;
	}
}



//initialization calls for opengl for static light
//note that we still need to enable lighting in order for it to work
//see keyboard 'l' event
void init_lighting() {

	if (lighting_mode == 0)
	{
		glDisable(GL_LIGHT1);
		glDisable(GL_LIGHT2);

		  glEnable(GL_LIGHT0);
  //set the light's position
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
  //set up the diffuse, ambient and specular components for the light
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);
  //specify our lighting model as 1 normal per face

	}
	else
	{

			  glDisable(GL_LIGHT0);


  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);
	
	//turn on light0
  //glEnable(GL_LIGHT0);
  //set the light's position
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glLightfv(GL_LIGHT1, GL_POSITION, light_pos1);
  //set up the diffuse, ambient and specular components for the light
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light_diff1);
  glLightfv(GL_LIGHT1, GL_AMBIENT, light_amb1);
  glLightfv(GL_LIGHT1, GL_SPECULAR, light_spec1);

  glLightfv(GL_LIGHT2, GL_POSITION, light_pos2);
  //set up the diffuse, ambient and specular components for the light
  glLightfv(GL_LIGHT2, GL_DIFFUSE, light_diff2);
  glLightfv(GL_LIGHT2, GL_AMBIENT, light_amb2);
  glLightfv(GL_LIGHT2, GL_SPECULAR, light_spec2);

	}
  
  //specify our lighting model as 1 normal per face

//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  
  if (display_mode == 0)
	glShadeModel(GL_FLAT);
  else
	glShadeModel(GL_SMOOTH);


  glEnable(GL_NORMALIZE);
}



int g_size;

void Draw3DSGrid()
{
	// This function was added to give a better feeling of moving around.
	// A black background just doesn't give it to ya :)  We just draw 100
	// green lines vertical and horizontal along the X and Z axis.

	// Turn the lines GREEN
	glColor3ub(0, 255, 0);

	// Draw a 1x1 grid along the X and Z axis'
	if (file_exists)
	for(float i = -1 * g_size; i <= g_size; i += 1)
	{
		// Start drawing some lines
		glBegin(GL_LINES);

			// Do the horizontal lines (along the X)
			glVertex3f(0 - g_size, 0, i);
			glVertex3f(g_size, 0, i);

			// Do the vertical lines (along the Z)
			glVertex3f(i, 0, -g_size);
			glVertex3f(i, 0, g_size);

		// Stop drawing lines
		glEnd();
	}
}

void renderBitmapString(
		float x, 
		float y, 
		float z, 
		void *font, 
		char *string) {  
  char *c;
  glRasterPos3f(x, y,z);
  //glRasterPos2f(x, y);
  for (c=string; *c != '\0'; c++) {
    glutBitmapCharacter(font, *c);
  }
}

static char strFrameRate[50] = {0};							

void CalculateFrameRate()
{

	static float framesPerSecond    = 0.0f;						
    static float lastTime			= 0.0f;						

    float currentTime = GetTickCount() * 0.001f;				
    ++framesPerSecond;
    if( currentTime - lastTime > 1.0f )
    {
	    lastTime = currentTime;
		
		sprintf(strFrameRate, "Current Frames Per Second: %d", int(framesPerSecond));

        framesPerSecond = 0;
    }
}

void drawFont()
{
	glColor3f(0.0, 0.0, 0.0);
	renderBitmapString(-1.5,1.4,0,GLUT_BITMAP_HELVETICA_18, strFrameRate);
	
}

void display() {

  float numV = Vertices.size();
  
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glMatrixMode(GL_MODELVIEW);

  glPushMatrix();
  glLoadIdentity();
  gluLookAt(ex, ey, ez, -cx, -cy, az, ux, uy, uz);
  //gluLookAt(-cx, -cy, ez, -cx, -cy, az, ux, uy, uz);
  glGetFloatv(GL_MODELVIEW_MATRIX, camM);
  glPopMatrix();
 
  if (!settings_change)
  {
	if ( mode == 1)
	{
			move();
	}
	else if (mode == 2)
	{
			zoom();
	}
	else if (mode == 3)
	{
		// translate mesh
			trans();
	}
	else if (mode == 4)
	{
		// scale mesh
			scale();
	}
	else if (mode == 5)
	{
		// rotate mesh
			rotate();

			startx = endx;
			starty = endy;
			glPushMatrix();
			glLoadIdentity();
			glRotatef(g_theta, rx, ry, rz);

			g_theta = 0;

			glMultMatrixf(trackballM);
			glGetFloatv(GL_MODELVIEW_MATRIX, trackballM);
			glPopMatrix();
	}
	else if (mode == 6)
	{ 	// translate light;
		   move_light();
		   init_lighting();
	}
	else if (mode == 7)
	{
		//rotate camera
		revolve();
	}
  }
  settings_change = false;

  glPushMatrix();
  glLoadIdentity();
  //set up the camera

  glMultMatrixf(camM);
      
  drawObjects();

    glPushMatrix();
  //glLoadIdentity();
  glTranslatef(0, t_ground, 0);
  Draw3DSGrid();
  glPopMatrix();

  glPushMatrix();
  glLoadIdentity();
   gluLookAt(0, 0, 6, 0, 0, az, ux, uy, uz);
   drawFont();
   glPopMatrix();

  glPopMatrix();
    
  glutSwapBuffers();
    
}

void reset()
{
  glPushMatrix();
  glLoadIdentity();
  glGetFloatv(GL_MODELVIEW_MATRIX, trackballM);
  glPopMatrix();
	
  display_mode = 1;
  //max_extent = 1.0;
  mode = 7;
  c_mode = 0;

  //lab 8 initializations
  startx = starty = endx = endy = 0.0;
  theta = 90;
  phi = 0;

  o_tx = o_ty = 0.0;
  tx = ty = 0.0;
  o_zx = o_zy = 0.0;
  zx = zy = 0.0;
  o_cx = o_cy = 0.0;
  cx = cy = 0.0;

  f_scale = 1;
  u_scale = 1;
  old_scale = 1;

  ex = 0.0;
  ey = 0.0;
  ez = 6.0;
  ax = 0.0;
  ay = 0.0;
  az = 0.0;
  ux = 0.0;
  uy = 1.0;
  uz = 0.0;

    ex = 6 * cos(phi) * cos(theta);
	ey = 6 * sin(phi);
	ez = 6 * cos(phi) * sin(theta);

  rx = ry = rz = 0.0;
  g_theta = 0.0;
  
  settings_change = false;
  first_pos = false;
  shape_done = true;
  draw_norms = false;

  light = 1;
  
  lx = ly = 1.0;
  o_lx = o_ly = 1.0;

 DRAW_TYPE = GL_POLYGON;
  
  //glDisable(GL_LIGHTING);
 		view_mode = 1;

		lighting_mode = 1;

	settings_change = true;
	reshape(GW, GH);
}

//mouse event function
void mouse(int button, int state, int x, int y) {
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) { /* if the left button is clicked */
      printf("mouse clicked at %d %d\n", x, GH-y-1);
      startx = x;
      starty = GH-y-1;
	  first_pos = true;
	  shape_done = false;

    } else if (state == GLUT_UP) {
      track = 0;
	  first_pos = false;
	  shape_done = true;
    }
  }
}

//mouse move function
void mouseMove(int x, int y) {
 printf("mouse moved at %d, %d\n", x, GH-y-1);
  endx = x;
  endy = GH-y-1;
  track = 1;

  //...TrackBall();
  glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y) {
  
  switch(key) {

	case 'h':
		printf("F - draw in polygons\n");
		printf("N - draw in normals\n");
		printf("L - toggle lighting\n");
		printf("M - toggle material\n");
		printf("V - toggle perspetive\n");
		printf("S - toggle smooth shading\n");
		printf("C - toggle cartoon rendering\n");
		printf("R - reset model\n");
		printf("Q - quit program\n");
		printf("\nTranslations and Transformations can accessed by right clicking");
	break;

  case 'l':
    light = !light;
    if (light)
	{  
		glEnable(GL_LIGHTING);
	  printf("lighting enabled\n");
	}
    else
	{
		glDisable(GL_LIGHTING);
		printf("lighting disabled\n");
	}
	settings_change = true;
	glutPostRedisplay();
    break;

  case 'c':
	if (lighting_mode == 0)
	{
		lighting_mode = 1;
	}
	else
		lighting_mode = 0;
	settings_change = true;
	init_lighting();
	glutPostRedisplay();
	break;

  case 'm':
    mat++;
    if (mat%2 == 0)
      materials(surface1);
    else if (mat%2 == 1)
      materials(surface2);
	
	settings_change = true;
	glutPostRedisplay();
    break;

  case 'f':
 	if (DRAW_TYPE == GL_LINE_LOOP)
	  DRAW_TYPE = GL_POLYGON;
	else
		DRAW_TYPE = GL_LINE_LOOP;
	settings_change = true;

	glutPostRedisplay();
	  break;

  case 's' :
	  	printf("toggling smooth shading\n");
	if (display_mode == 0)
		display_mode = 1;
	else
		display_mode = 0;
	settings_change = true;
	glutPostRedisplay();
	init_lighting();
	break;


  case 'r':
	  reset();
	  break;




  case 'v':
    printf("toggling view mode\n");
    if (view_mode == 0)
		view_mode = 1;
	else
		view_mode = 0;

	settings_change = true;
	reshape(GW, GH);

    break;
  case 'n':
	  draw_norms = !draw_norms;
	  settings_change = true;
	  glutPostRedisplay();
	  break;

  case 'q': case 'Q' :
    exit( EXIT_SUCCESS );
    break;

	
  default:
    break;
  }
  
}



void mymenu (int value)
{


	if (value == 4)
	{
		if (view_mode == 0)
		view_mode = 1;
	else
		view_mode = 0;

	settings_change = true;
	reshape(GW, GH);
	}
	if (value == 5)
	{
		DRAW_TYPE = GL_LINE_LOOP;
		settings_change = true;

	glutPostRedisplay();
	}
	if (value == 6)
	{
		DRAW_TYPE = GL_POLYGON;
		settings_change = true;

	glutPostRedisplay();
	}
	if (value == 7)
	{
	  draw_norms = !draw_norms;
	  settings_change = true;
	  glutPostRedisplay();

	}
	if (value == 8)
	{
		reset();
		settings_change = true;
		glutPostRedisplay();
	}
	
	if (value == 9) exit(0);
	
	settings_change = true;
}

	void camera_menu(int value)
	{
		if (value == 1)
		{
			//translate camera
			mode = 1;
		}
		if (value == 2)
		{
			//zoom camera
			mode = 2;
		}
		if (value == 3)
		{
			//rotate camera
			mode = 7;
		}

		settings_change = true;
	}
	
	void mesh_menu(int value)
	{		
		if (value == 1)
		{
			//translate mesh
			mode = 3;
		}
		if (value == 2)
		{
			//scale mesh
			mode = 4;
		}
		if (value == 3)
		{
			//rotate mesh
			mode = 5;
		}
		settings_change = true;
	}
	
	void light_menu(int value)
	{
		if (value == 1)
		{
			//translate light
			mode = 6;
			settings_change = true;
		}
		settings_change = true;
		if (value == 2)
		{
			printf("toggling smooth shading\n");
			if (display_mode == 0)
				display_mode = 1;
			else
				display_mode = 0;
			settings_change = true;
			glutPostRedisplay();
			init_lighting();
		}
		if (value == 4)
		{
			//enable light
			    light = !light;
			if (light)
			{  
				glEnable(GL_LIGHTING);
			printf("lighting enabled\n");
			}
			else
			{
				glDisable(GL_LIGHTING);
				printf("lighting disabled\n");
			}
			settings_change = true;
			glutPostRedisplay();

		}
		if (value == 3)
		{
			//toggle material
			    mat++;
			if (mat%2 == 0)
			materials(surface1);
			else if (mat%2 == 1)
			materials(surface2);
			
			settings_change = true;
			glutPostRedisplay();
		}
		if (value == 5)
		{
		if (lighting_mode == 0)
		{
			lighting_mode = 1;
		}
		else
			lighting_mode = 0;
		settings_change = true;
		init_lighting();
		glutPostRedisplay();
		}

	}

int main( int argc, char** argv ) {
  
  //set up my window
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowSize(400, 400); 
  glutInitWindowPosition(200, 200);
  glutCreateWindow("Mesh display");
  glClearColor(1.0, 1.0, 1.0, 1.0);
  
  //register glut callback functions
  glutDisplayFunc( display );
  glutReshapeFunc( reshape );
  glutKeyboardFunc(keyboard);
  glutMouseFunc(mouse);
  glutMotionFunc(mouseMove);
  //enable z-buffer
  glEnable(GL_DEPTH_TEST);
  
  //set-up viewing
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-2.0, 2.0, -2.0, 2.0, 1.0, 15.0);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, 300, 300); //image to pixel
  
  //initialization
  max_x = max_y = max_z = -999999;
  min_x = min_y = min_z = 99999;
  center.x = 0;
  center.y = 0;
  center.z = 0;
  display_mode = 1;
  max_extent = 1.0;
  view_mode = 1;
  mode = 7;

  lighting_mode = 1;

  c_mode = 0;

  //lab 8 initializations
  startx = starty = endx = endy = 0.0;

  o_tx = o_ty = 0.0;
  tx = ty = 0.0;
  o_zx = o_zy = 0.0;
  zx = zy = 0.0;
  o_cx = o_cy = 0.0;
  cx = cy = 0.0;

  theta = 90;
  phi = 0;


  ex = 0.0;
  ey = 0.0;
  ez = 6.0;
  ax = 0.0;
  ay = 0.0;
  az = 0.0;
  ux = 0.0;
  uy = 1.0;
  uz = 0.0;
    ex = 6 * cos(phi) * cos(theta);
	ey = 6 * sin(phi);
	ez = 6 * cos(phi) * sin(theta);

  rx = ry = rz = 0.0;
  g_theta = 0.0;
  
  settings_change = false;
  first_pos = false;
  shape_done = true;
  draw_norms = false;

  f_scale = 1;
  u_scale = 1;
  old_scale = 1;

  
  light = 1;
  
  //enable GL features we want
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  init_lighting();

  lx = ly = 1.0;
  o_lx = o_ly = 1.0;

  cool.x = -1.0;
  cool.y = -1.0;
  cool.z = 1.0;

   DRAW_TYPE = GL_POLYGON;
   
  file_exists = false;

  
	  
  //make sure a file to read is specified
  if (argc > 1) {
	  
	  file_exists = true;
    cout << "file " << argv[1] << endl;
    //read-in the mesh file specified
    ReadFile(argv[1]);

	    
    //now that the file has been parsed, set up the edge data structure
      computeEdge();

    
    //once the file is parsed find out the maximum extent to center and scale mesh
    max_extent = max_x - min_x;
    if (max_y - min_y > max_extent) max_extent = max_y - min_y;
    //cout << "max_extent " << max_extent << " max_x " << max_x << " min_x " << min_x << endl;
    //cout << "max_y " << max_y << " min_y " << min_y << " max_z " << max_z << " min_z " << min_z << endl;
    
    center.x = center.x/Vertices.size();
    center.y = center.y/Vertices.size();
    center.z = center.z/Vertices.size();
    //cout << "center " << center.x << " " << center.y << " " << center.z << endl;
    //cout << "scale by " << 1.0/(float)max_extent << endl;
  } else {
    cout << "format is: meshparcer filename" << endl;
  }





  int menu1, menu_mesh, menu_camera, menu_light;

  menu_mesh = glutCreateMenu(mesh_menu);
  glutAddMenuEntry("Move", 1);
  glutAddMenuEntry("Scale", 2);
  glutAddMenuEntry("Rotate", 3);
  
  menu_camera = glutCreateMenu(camera_menu);
  //glutAddMenuEntry("Move", 1);
  //glutAddMenuEntry("Zoom", 2);
  glutAddMenuEntry("Rotate", 3);

  menu_light = glutCreateMenu(light_menu);
  glutAddMenuEntry("Toggle Lighting", 4);
  glutAddMenuEntry("Toggle Cartoon Rendering", 5);
  //glutAddMenuEntry("Move Light Source", 1);
  glutAddMenuEntry("Toggle Smooth Shading", 2);
  //glutAddMenuEntry("Switch Material", 3);

  menu1 = glutCreateMenu(mymenu);
  glutAddSubMenu("Camera", menu_camera);
  glutAddSubMenu("Mesh", menu_mesh);
  glutAddSubMenu("Light", menu_light);
  //glutAddMenuEntry("Toggle Perspective", 4);

  //glutAddMenuEntry("Wireframe", 5);
  //glutAddMenuEntry("Filled Polygons", 6);
  //glutAddMenuEntry("Normals", 7);
  glutAddMenuEntry("Reset", 8);
  glutAddMenuEntry("Exit", 9);
  glutAttachMenu(GLUT_RIGHT_BUTTON);

 
  glutMainLoop();
}


